
/*+*******************************************************************

 File:		DevServer.h

 Project:	Device Servers

 Description:	Public include for for applications wanting to
		to use device servers.

 Author(s):	Andy Goetz
		Jens Meyer

 Original:	March 1990


 $Revision: 8.18 $
 $Date: 2000/11/12 13:24:04 $

 $Author: goetz $

 $Log:	DevServer.h,v $
 * Revision 8.18  2000/11/12  13:24:04  13:24:04  goetz (Andy Goetz)
 * added dynamic error handling; improved TANGO error treatment
 * 
 Revision 8.17  1900/09/26 13:36:40  goetz
 tango_dev_putget() and tango_dev_putget_raw() implement immediate reconnection

 * Revision 8.16  2000/09/25  14:50:18  14:50:18  goetz (Andy Goetz)
 * tango_api implements TANGO_HOST, stateless import and reimport
 * 
 * Revision 8.15  20/0./7.  1.:6.:9.  1.:6.:9.  goetz (Andy Goetz)
 * changed from tango database to host mysql, added support for float and double arrays
 * 
 * Revision 8.14  20/0./6.  2.:4.:0.  2.:4.:0.  goetz (Andy Goetz)
 * fixed bug in dev_event_listen() which caused server to crash
 * 
 Revision 8.13  2000/06/02 21:56:55  goetz
 _DEVICE_H now _TANGO_DEVICE_H in Device.H; (char*)taco_tango

 Revision 8.12  2000/06/02 15:58:13  goetz
 dev_event_fire() now device specific; ported to SuSE V6.4

 Revision 8.11  2000/05/31 07:48:00  goetz
 tango_api has local copies of argc and argv, ported to HP-UX

 * Revision 8.10  2000/05/29  21:39:25  21:39:25  goetz (Andrew GOETZ)
 * fixed prototyping problems with C++ on HP-UX
 * 
 * Revision 8.9  2000/05/29  18:13:35  18:13:35  goetz (Andrew GOETZ)
 * fixed bug in dev_import_timeout() which prevented timeouts < 2s
 * 
 Revision 8.8  2000/05/02 15:32:20  goetz
 added prototype for tsleep() for OS9 C++

 * Revision 8.7  2000/05/02  14:19:41  14:19:41  goetz (Andy Goetz)
 * removed check for device server running on different host
 * 
 * Revision 8.6  2000/05/02  13:29:35  13:29:35  goetz (Andy Goetz)
 * added random sleep to gettransient + 3 retries
 * 
 Revision 8.5  2000/03/27 17:13:48  goetz
 declared pmap_getmaps() as external C function for OS9 C++

 Revision 8.4  2000/03/13 14:50:05  goetz
 import timeout now programmable using dev_import_timeout()

 Revision 8.3  2000/03/13 10:31:38  goetz
 added Jens' modifications to fix bug in local putget

 Revision 8.2  2000/03/10 17:01:26  goetz
 dev_synch() now excludes udp clients and new imports

 Revision 8.1  2000/01/18 16:48:54  goetz
 tango_dev_import() now only called in dev_import if -DTANGO

 Revision 8.0  1999/12/28 14:18:35  goetz
 added TANGO support for TACO dev_xxx() calls via -DTANGO for C++

 Revision 7.10  1999/11/25 08:27:16  goetz
 replaced fprintf(stderr) with printf; made startup() C++ compatible

 Revision 7.9  1999/11/22 20:19:02  goetz
 removed const from gettransient() prototype

 Revision 7.8  1999/11/21 20:45:39  goetz
 included all M.Diehl's patches (major changes to gettransient() + main())

 Revision 7.7  1999/11/18 22:29:04  goetz
 fixed event multi-client bug, free client handle after event_listen, timeout in dev_synch()

 Revision 7.6  1999/11/01 19:05:17  goetz
 make local copy of user name returned by getpwuid() in sec_api.c

 Revision 7.5  1999/08/06 17:44:18  goetz
 removed usage of varargs for g++ on solaris and hpux

 * Revision 7.4  99/07/09  05:15:29  05:15:29  goetz (Andy Goetz)
 * added M.Diehl's patch to DevServerSig.c to exit() after calling unregister_server()
 * 
 Revision 7.3  1999/06/07 15:27:06  goetz
 fixed bug with multi-nethost reimport, device name stored with nethost

 Revision 7.2  1999/05/12 15:22:09  goetz
 changed dev_event_fire() to void; fixed bug in dev_event_unlisten()

 * Revision 7.1  99/05/11  15:59:56  15:59:56  goetz (Andy Goetz)
 * replace static declaration of event_client[] array by malloc()
 * 
 Revision 7.0  1999/04/26 07:30:47  goetz
 implemented user events (added event_api.c)

 Revision 6.15  1999/03/27 09:00:26  goetz
 asynch_cleanup() only checks for asynch clients; C++ returns output arguments if DS_WARNING

 * Revision 6.14  99/02/27  15:21:26  15:21:26  goetz (Andy Goetz)
 * fixed reimport bug; disabled reimport for rpc version 1
 * 
 Revision 6.13  1999/02/05 00:50:33  goetz
 fixed reimport socket+memory leak; ported to aCC on HPUX; suppressed rpc errors

 * Revision 6.12  99/01/18  21:39:10  21:39:10  goetz (Andy Goetz)
 * suppressed test for RPC_TIMEDOUT in svc_check()
 * 
 * Revision 6.11  99/01/14  23:15:00  23:15:00  goetz (Andy Goetz)
 * dbase timeout = 4 s; unregister ver. 1; svc_check() handles RPC_TIMEDOUT
 * 
 * Revision 6.10  98/12/22  10:29:52  10:29:52  taurel (E.Taurel)
 * Added support for Solaris and HP C++ native compiler. Remove K_R C support
 * 
 * Revision 6.9  98/12/15  14:05:13  14:05:13  goetz (Andy Goetz)
 * unregister RPC; use pgm no. 1 as flag; dev_pending() checks timeout; extern C prototype
 * 
 * Revision 6.8  98/11/20  14:35:41  14:35:41  goetz (Andy Goetz)
 * unregister API_VERSION (4) correctly now + check return value of svc_register()
 * 
 * Revision 6.7  98/11/20  10:35:07  10:35:07  goetz (Andy Goetz)
 * improved gettrans_ut(); multi-config_flags=0; removde static client_data; fixed linux select() bug; only register pgm # 0,4,5; portmap address == 111
 * 
 * Revision 6.6  98/09/24  09:25:14  09:25:14  goetz (Andy Goetz)
 * fixed stupid type which reintroduced asynchronous memory leak
 * 
 * Revision 6.5  98/08/26  16:20:38  16:20:38  goetz (Andy Goetz)
 * added dynamic NETHOST allocation, fixed bug in db_setup_multi, ported to Irix 6.4
 * 
 * Revision 6.4  98/04/29  10:21:27  10:21:27  goetz (Andy Goetz)
 * asynchronous - memory leak fixed, OS9 bug fixed, stale handle cleanup, pending=300
 * 
 * Revision 6.3  97/12/10  15:28:37  15:28:37  meyer (J.Meyer)
 * Changed select() timeout in rpc_check_host() to 200ms
 * 
 * Revision 6.2  97/12/01  12:59:20  12:59:20  goetz (Andy Goetz)
 * intermediate checkin before opening 5.14.1 branch
 * 
 * Revision 6.1  97/11/20  21:49:30  21:49:30  goetz (Andy Goetz)
 * main() continue even if cannot register asynch rpc
 * 
 * Revision 6.0  97/11/03  17:22:31  17:22:31  goetz (Andy Goetz)
 * merged synchronous and asynchronous branches
 * 
 Revision 5.15  1997/10/24 15:27:07  klotz
 ported to Windows NT 4.0 + NMSDK 5.0 ; tested with TextTalkds ; new DevServerMain.c

 * Revision 5.13  97/09/03  17:34:37  17:34:37  meyer (J.Meyer)
 * Added casting for ucc++ in DevServerSig.c
 * 
 * Revision 5.12  97/09/03  15:54:13  15:54:13  meyer (J.Meyer)
 * Changed OS9 signal handling from intercept() to signal().
 * 
 * Revision 5.11  97/08/05  14:19:32  14:19:32  goetz (Andy Goetz)
 * ported to VxWork 5.2, pgm. no. based on CRC, -m suppressed, -ve errors bug fix
 * 
 * Revision 5.10  97/03/14  17:26:13  17:26:13  dserver ()
 * fixed bug with open file handles after free
 * 
 * Revision 5.8.1.4  97/03/14  17:24:53  17:24:53  goetz (Andy Goetz)
 * fixed bug with open file handles after free
 * 
 * Revision 5.9  97/03/13  11:17:15  11:17:15  klotz (W.D. Klotz)
 * first part of NT4.0 port
 * 
 * Revision 5.8  97/01/31  12:06:25  12:06:25  goetz (Andy Goetz)
 * dev_protocol(), Linux+security bug fixes, dev_import() ignores timeout on NULL procedure, svc_run for OS9
 * 
 * Revision 5.7  97/01/14  08:57:21  08:57:21  goetz (Andy Goetz)
 * added stateless to dev_free(),_raw(),_asyn(),_protocol(),_timeout()
 * 
 * Revision 5.6  97/01/12  18:33:41  18:33:41  goetz (Andy Goetz)
 * dev_import() now stateless i.e. import works even if server is not running
 * 
 * Revision 5.5  97/01/08  17:02:30  17:02:30  goetz (Andy Goetz)
 * Ported to Linux (2.0.9)
 * 
 * Revision 5.4  96/11/27  09:58:16  09:58:16  goetz (Andy Goetz)
 * removed rpc admin interface from DevServerMain.c; putenv() suppressed for _UCC
 * 
 * Revision 5.3  96/11/26  16:24:42  16:24:42  goetz (Andy Goetz)
 * fixed bug (i_nethost=0) in dev_import_local()
 * 
 * Revision 5.2  96/11/26  13:31:05  13:31:05  goetz (Andy Goetz)
 * OICDevice (C++ wrapper for OIC) added + fixed multi-nethost memory leaks
 * 
 * Revision 5.1  96/11/04  10:55:23  10:55:23  goetz (Andy Goetz)
 * added multi nethost support
 * 

 Copyright (c) 1990-1997 by European Synchrotron Radiation Facility, 
                            Grenoble, France

********************************************************************-*/

#ifndef _DevServer_h
#define _DevServer_h

#include <API.h>

#ifdef __cplusplus
#include <Device.H>
#endif /* __cplusplus */

/*
 *  standart header string to use "what" or "ident".
 */
#ifdef _IDENT
static char DevServerh[] =
"@(#)$Header: DevServer.h,v 8.18 2000/11/12 13:24:04 goetz Rel $";
#endif /* _IDENT */


typedef long DevMethod;


#define Fontaine_ps 	1
#define WDK_ps 		2
#define Ritz_ps		3
#define Pirani_gauge 	4
#define Penning_gauge 	5
#define Fluor_screen	6
#define Vacuum_gauges	7
#define Dan8000_ps	8


/*
 * standard methods, available in every class
 */

#define DevMethodCreate			1
#define DevMethodDestroy		2
#define DevMethodClassInitialise	3
#define DevMethodStateHandler		4

/*
 * DevServerClass methods
 */

#define DevMethodCommandHandler		10
#define DevMethodErrorHandler		11
#define DevMethodInitialise		12
#define DevMethodDevExport		13

/*
 * Device dependent Methods
 */

#define DevMethodLineOpen		101
#define DevMethodLineClose		102
#define DevMethodWrite			103
#define DevMethodRead			104
#define DevMethodWriteCommand		105
#define DevMethodReadAnswer		106
#define DevMethodReadError		107
#define DevMethodControllerInitialise	108
#define DevMethodChangeCalib		109
#define DevMethodSetProtThresh		110
#define DevMethodResetInputBuffer	111
#define DevMethodDev_Diagnostics	112


/*
 * miscellaneous methods
 */

#define DevMethodHelloWorld		100

#ifndef __cplusplus
typedef long 	int(*DevMethodFunction)();
#else
typedef long 	int(*DevMethodFunction)(...);
#endif /* __cplusplus */

typedef struct _DevMethodListEntry {
	DevMethod method;
	DevMethodFunction fn;
	}	DevMethodListEntry;

typedef struct _DevMethodListEntry *DevMethodList;


typedef struct _DevServerClassRec *DevServerClass;
typedef struct _DevServerRec *DevServer;

#ifndef __cplusplus
extern DevServerClass devServerClass;
#else
extern "C" DevServerClass devServerClass;
#endif

/*
 * typedefs for the device access structures
 * with security.
 */

typedef struct _DevServerAccess {
#ifndef __cplusplus
/*
 * OIC version
 */
	DevServer 		ds;
#else
/*
 * C++ version
 */
	Device			*device;
#endif
	char  			export_name[LONG_NAME_SIZE];
	long 			export_status;
	long 			export_counter;
	long    		single_user_flag;
	long    		admin_user_flag;
	long			si_client_id;
	long			si_access_right;
	long			si_connection_id;
	long			si_tcp_socket;
	struct sockaddr_in 	si_peeraddr;
	} DevServerDevices;

/*
 * Include security system definitions
 */

#include <DevSec.h>

/*
 * the method finder, indispensable for OOPS
 */

#ifdef __cplusplus
extern "C" {
#endif
extern DevMethodFunction ds__method_finder
				PT_( (void *ptr_ds, DevMethod method) );

extern long ds__method_search  	PT_( (void *ptr_ds_class, DevMethod method,
				DevMethodFunction *function_ptr) );
extern long ds__create          PT_( (char *name, void *ptr_ds_class,
				void *ptr_ds_ptr, long *error) );
extern long ds__destroy         PT_( (void *ptr_ds, long *error) );
extern long ds__svcrun          PT_( (long *error) );
extern long startup  		PT_( (char* svr_name,long* error) );


#ifdef __cplusplus
	}
#endif /* __cplusplus */

extern long ds__signal          PT_( (int sig, void (*action)(), long *error) );
extern int gettransient         PT_( (const char *ds_name) );
extern void main_signal_handler PT_( (int sig) );
extern void unregister_server   PT_( (void) );

/*
 * Convenience functions for the Device Server programs. These provide 
 * and alternate calling sequence for executing methods of the root 
 * class. 
 */

#ifdef __cplusplus
extern "C" long dev_cmd PT_( (void *ptr_ds, long cmd, DevArgument argin,
			long argin_type, DevArgument argout, long argout_type,
			long *error) );
#else
extern long dev_cmd 	PT_( (void *ptr_ds, long cmd, DevArgument argin,
			long argin_type, DevArgument argout, long argout_type,
			long *error) );
#endif /* __cplusplus */

#ifndef __cplusplus
/*
 * OIC version
 */
extern long dev_export  PT_( (char *dev_name, void *ptr_ds, long *error) );
extern void dev_event_fire 
				PT_( (DevServer ds, long event_type,
				DevArgument argout, DevType argout_type,
				long event_status, long event_error) );
#else
/*
 * C++ version
 */
extern "C" long dev_export  PT_( (char* dev_name, Device *ptr_dev, long *error) );
extern "C" void dev_event_fire 
				PT_( (Device *device, long event_type,
				DevArgument argout, DevType argout_type,
				long event_status, long event_error) );
#endif /* __cplusplus */

/*
 * NT stuff for DevServerMain
 */
#ifdef _NT
extern HWND hMainWnd;         /* handle of app's main window */
extern HANDLE hMainInstance;  /* handle of app's module instance */
extern char* szAppName;       /* app's name */
/* 
 * Function called from 'libdsapi' for delayed startup. Useful for
 * Windows applications to perform startup operations when Window's
 * GUI has been initialized.
 */
extern long (*DelayedStartup)();  
#endif  /* _NT */
#endif /* _DevServer_h */
